package ukol;

import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.ResultSet;
import java.sql.ResultSetMetaData;
import java.sql.SQLException;
import java.sql.Statement;

import javax.swing.table.AbstractTableModel;

/**
 * Model pro komponentu JTable zapouzdřující výsledek SQL dotazu ResultSet a
 * umožňující načíst výsledky z SQL databáze pro určitý dotaz.
 */
public class ResultSetTableModel extends AbstractTableModel {

    /**
     * Spojení s databází.
     */
    private Connection connection;
    /**
     * Statement pro provádění SQL dotazů.
     */
    private Statement statement;
    /**
     * Výsledek dotazu.
     */
    private ResultSet resultSet;
    /**
     * Meta data výsledku dotazu.
     */
    private ResultSetMetaData metaData;
    /**
     * Počet řádků výsledku.
     */
    private int numberOfRows;
    /**
     * Stav připojení k databázi.
     */
    private boolean connectedToDatabase = false;

    /**
     * Vytvoří novou instanci třídy ResultSetTableModel.
     *
     * @param driver
     *            plně kvalifikované jméno použitého JDBC ovladače.
     * @param url
     *            url připojení k databázi.
     * @param username
     *            uživatelské jméno použité pro přístup k databázi.
     * @param password
     *            heslo použité pro přístup k databázi.
     * @throws ClassNotFoundException
     *             pokud nebude JDBC ovladač driver umístěný v classpath.
     * @throws SQLException
     *             pokud dojde při zpracování SQL dotazu k chybě.
     */
    public ResultSetTableModel(String driver, String url, String username,
            String password) throws ClassNotFoundException,
            SQLException {
        // načtení ovladače databáze
        Class.forName(driver);

        // připojení k databázi
        connection = DriverManager.getConnection(url, username, password);

        // vytvoření objektu Statement pro provádění dotazů
        statement =
                connection.createStatement(ResultSet.TYPE_SCROLL_INSENSITIVE,
                ResultSet.CONCUR_READ_ONLY);

        // aktualizace stavu připojení
        connectedToDatabase = true;
    }

    public Connection getConnection() {
        return connection;
    }

    @Override
    public Class<?> getColumnClass(int column) {
        if (!connectedToDatabase) {
            throw new IllegalStateException("Nepřipojeno k databázi.");
        }

        try {
            String className = metaData.getColumnClassName(column + 1);
            return Class.forName(className);
        } catch (Exception e) {
            e.printStackTrace();
        }

        // pokud dojde k problémům, předpokládáme třídu Object
        return Object.class;

    }

    @Override
    public int getRowCount() {
        if (!connectedToDatabase) {
            throw new IllegalStateException("Nepřipojeno k databázi.");
        }

        return numberOfRows;
    }

    @Override
    public int getColumnCount() {
        if (!connectedToDatabase) {
            throw new IllegalStateException("Nepřipojeno k databázi.");
        }
        if (metaData == null) { // nebyl proveden SQL dotaz
            return 0;
        }
        try {
            return metaData.getColumnCount();
        } catch (SQLException e) {
            e.printStackTrace();
        }

        // pokud dojde k problémům, vrátíme 0
        return 0;
    }

    @Override
    public String getColumnName(int column) {
        if (!connectedToDatabase) {
            throw new IllegalStateException("Nepřipojeno k databázi.");
        }

        try {
            // sloupce jsou číslovány od 0 v JTable,
            // ale od 1 v ResultSetu
            //return metaData.getColumnName(column + 1);
            return metaData.getColumnLabel(column + 1);
        } catch (SQLException e) {
            e.printStackTrace();
        }

        // pokud dojde k problémům, vrátíme prázdný řetězec
        return "";
    }

    @Override
    public Object getValueAt(int rowIndex, int columnIndex) {
        if (!connectedToDatabase) {
            throw new IllegalStateException("Nepřipojeno k databázi.");
        }

        try {
            // nastavení řádku
            // řádky jsou číslovány od 0 v JTable,
            // ale od 1 v ResultSetu
            resultSet.absolute(rowIndex + 1);
            // vrácení hodnoty v daném sloupci
            // sloupce jsou číslovány od 0 v JTable,
            // ale od 1 v ResultSetu
            return resultSet.getObject(columnIndex + 1);
        } catch (SQLException e) {
            e.printStackTrace();
        }

        // pokud dojde k problémům, vrátíme prázdný řetězec
        return "";
    }

    /**
     * Nastaví výsledek obsažený v tomto modelu podle zadaného dotazu.
     *
     * @param result
     *            výsledek dotazu, jenž bude obsažen v tomto modelu.
     * @throws SQLException
     *             pokud při zpracování dotazu dojde k chybě.
     */
    public void setResult(ResultSet result) throws SQLException {
        if (!connectedToDatabase) {
            return;
        }
        if (result == null) {
            return;
        }
        resultSet = result;

        // získání meta dat
        metaData = resultSet.getMetaData();

        // určení počtu řádků v ResultSetu
        resultSet.last();
        numberOfRows = resultSet.getRow();

        // upozornění JTable, že se model změnil
        fireTableStructureChanged();
    }

    /**
     * Provede odpojení od databáze a uvolnění prostředků.
     */
    public void disconnectFromDatabase() {
        if (!connectedToDatabase) {
            return;
        }

        try {
            statement.close();
            connection.close();
        } catch (SQLException e) {
            e.printStackTrace();
        } finally {
            connectedToDatabase = false;
        }
    }

    public String toHtml() {
        StringBuffer s = new StringBuffer();
        s.append("<table border=\"1\" class=\"resultSet\" />");
        s.append("<tr>");
        for(int i = 0; i < getColumnCount(); i++) {
            s.append("<th>").append(getColumnName(i)).append("</th>");
        }
        s.append("</tr>\n");
         for(int r = 0; r < getRowCount(); r++) {
            s.append("<tr>");
            for(int c = 0; c < getColumnCount(); c++) {
            s.append("<td>").append(getValueAt(r,c)).append("</td>");
        }
            s.append("</tr>");
         }
        
        s.append("</table>");
        return s.toString();
    }
}
